<?php

/**
 * Contains \Drupal\robokassa_payment\Plugin\Payment\Method\Robokassa.
 */

namespace Drupal\robokassa_payment\Plugin\Payment\Method;


use Drupal\Core\Access\AccessResult;
use Drupal\Core\Session\AccountInterface;
use Drupal\payment\Entity\Payment;
use Drupal\payment\Entity\PaymentInterface;
use Drupal\payment_offsite_api\Plugin\Payment\Method\PaymentMethodOffsiteSimple;
use Drupal\payment_offsite_api\Plugin\Payment\Method\PaymentMethodOffsiteSimpleInterface;

/**
 * A basic payment method that does not transfer money.
 *
 * Plugins that extend this class must have the following keys in their plugin
 * definitions:
 * - entity_id: (string) The ID of the payment method entity the plugin is for.
 * - execute_status_id: (string) The ID of the payment status plugin to set at
 *   payment execution.
 *
 * @PaymentMethod(
 *   id = "payment_robokassa",
 *   deriver = "\Drupal\payment_offsite_api\Plugin\Payment\Method\PaymentMethodBaseOffsiteDeriver",
 *   operations_provider = "\Drupal\payment_offsite_api\Plugin\Payment\Method\PaymentMethodBaseOffsiteOperationsProvider"

 * )
 */
class Robokassa extends PaymentMethodOffsiteSimple implements PaymentMethodOffsiteSimpleInterface {

  /**
   * Robokassa server payment URL.
   */
  const SERVER_URL = 'https://auth.robokassa.ru/Merchant/Index.aspx';
  const TEST_SERVER_URL = 'https://auth.robokassa.ru/Merchant/Index.aspx';

  /**
   * {@inheritdoc}
   */
  public function getSupportedCurrencies() {
    return TRUE;
  }

  /**
   * Gets the ID of the payment method this plugin is for.
   *
   * @return string
   */
  public function getEntityId() {
    return $this->pluginDefinition['entity_id'];
  }

  public function getServer() {
    return $this->pluginDefinition['config']['server'];
  }

  public function getAction() {
    return $this->getServer() == 'test' ? self::TEST_SERVER_URL : self::SERVER_URL;
  }

  public function getMerchant() {
    return $this->pluginDefinition['config']['MrchLogin'];
  }

  public function getPass1() {
    return $this->pluginDefinition['config']['pass1'];
  }

  public function getPass2() {
    return $this->pluginDefinition['config']['pass2'];
  }

  public function getHashType() {
    return $this->pluginDefinition['config']['hash_type'];
  }

  /**
   * {@inheritdoc}
   */
  public function updatePaymentStatusAccess(AccountInterface $account) {
    return AccessResult::allowed();
  }

  /**
   * {@inheritdoc}
   */
  public function getSettablePaymentStatuses(AccountInterface $account, PaymentInterface $payment) {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function getResultPages() {
    return [
      'success' => FALSE,
      'fail' => TRUE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function paymentForm() {
    $form = array();
    $payment = $this->getPayment();
    $form['#action'] = $this->getAction();

    $this->addPaymentFormData('MrchLogin', $this->getMerchant());
    $this->addPaymentFormData('InvId', $payment->id());
    $this->addPaymentFormData('OutSum', round($payment->getAmount(), 2));
    $this->addPaymentFormData('SignatureValue', $this->getSignature());

    // For test transactions.
    if ($this->getServer() == 'test') {
      $this->addPaymentFormData('IsTest', '1');
    }

    $form += $this->generateForm();

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getSignature($signature_type = PaymentMethodOffsiteSimple::SIGN_OUT) {
    $payment = $this->getPayment();
    $signature_data = array(
      $this->getMerchant(),
      $payment->getAmount(),
      $payment->id(),
    );

    $signature_data[] = $signature_type == PaymentMethodOffsiteSimple::SIGN_OUT ? $this->getPass1() : $this->getPass2();

    // Calculate signature.
    return hash($this->getHashType(), implode(':', $signature_data));
  }

  /**
   * Return local payment status related to given remote.
   *
   * @param string $status
   *   Remote status id.
   *
   * @return string.
   *   Local status id.
   */
  public function getStatusId($status) {
    return $this->pluginDefinition['ipn_statuses'][$status];
  }

  /**
   * {@inheritdoc}
   */
  public function ipnExecute() {
    if (!$this->ipnValidate()) {
      return [
        'status' => 'fail',
        'message' => 'bad data',
        'response_code' => 200,
      ];
    }

    $inv_id = $this->request->request->get('InvId');
    $payment_status = $this->getStatusId($inv_id);
    $status = isset($payment_status) ? $payment_status : 'payment_pending';
    $this->getPayment()->setPaymentStatus($this->paymentStatusManager->createInstance($status));
    $this->getPayment()->save();

    return [
      'status' => 'success',
      'message' => 'OK' . $this->getPayment()->id(),
      'response_code' => 200,
    ];
  }

  /**
   * Returns success message.
   *
   * @return array
   *   The renderable array.
   */
  public function getSuccessContent() {
    return [
      '#markup' => $this->t('Payment processed as @external_status.', array('@external_status' => 'success')),
    ];
  }

  /**
   * Returns fail message.
   *
   * @return array
   *   The renderable array.
   */
  public function getFailContent() {
    return [
      '#markup' => $this->t('Payment processed as @external_status.', array('@external_status' => 'FAIL')),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getMerchantIdName() {
    return 'MrchLogin';
  }

  /**
   * {@inheritdoc}
   */
  public function getTransactionIdName() {
    return 'InvId';
  }

  /**
   * {@inheritdoc}
   */
  public function getAmountName() {
    return 'OutSum';
  }

  /**
   * {@inheritdoc}
   */
  public function getSignatureName() {
    return 'SignatureValue';
  }

  /**
   * {@inheritdoc}
   */
  public function getRequiredKeys() {
    $required_keys = [
      'OutSum',
      'InvId',
    ];
    if (!$this->isFallbackMode()) {
      $required_keys[] = 'SignatureValue';
    }

    return $required_keys;
  }

  /**
   * {@inheritdoc}
   */
  public function isConfigured() {
    return !empty($this->getAction());
  }
}
